@@ -1,5 +1,24 @@
+#  Copyright 2021 Collate
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#  http://www.apache.org/licenses/LICENSE-2.0
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+"""
+OpenMetadata is the high level Python API that serves as a wrapper
+for the metadata-server API. It is based on the generated pydantic
+models from the JSON schemas and provides a typed approach to
+working with OpenMetadata entities.
+"""
+
+import asyncio
 import logging
-from typing import Generic, List, Optional, Type, TypeVar, Union, get_args
+import urllib
+from typing import Dict, Generic, List, Optional, Type, TypeVar, Union, get_args
 
 from pydantic import BaseModel
 
@@ -7,24 +26,31 @@ from metadata.generated.schema.api.lineage.addLineage import AddLineage
 from metadata.generated.schema.entity.data.chart import Chart
 from metadata.generated.schema.entity.data.dashboard import Dashboard
 from metadata.generated.schema.entity.data.database import Database
+from metadata.generated.schema.entity.data.location import Location
 from metadata.generated.schema.entity.data.metrics import Metrics
-from metadata.generated.schema.entity.data.mlmodel import MLModel
-from metadata.generated.schema.entity.data.model import Model
+from metadata.generated.schema.entity.data.mlmodel import MlModel
 from metadata.generated.schema.entity.data.pipeline import Pipeline
 from metadata.generated.schema.entity.data.report import Report
 from metadata.generated.schema.entity.data.table import Table
 from metadata.generated.schema.entity.data.topic import Topic
+from metadata.generated.schema.entity.policies.policy import Policy
 from metadata.generated.schema.entity.services.dashboardService import DashboardService
 from metadata.generated.schema.entity.services.databaseService import DatabaseService
 from metadata.generated.schema.entity.services.messagingService import MessagingService
 from metadata.generated.schema.entity.services.pipelineService import PipelineService
-from metadata.generated.schema.entity.tags.tagCategory import Tag
+from metadata.generated.schema.entity.services.storageService import StorageService
+from metadata.generated.schema.entity.tags.tagCategory import Tag, TagCategory
+from metadata.generated.schema.entity.teams.role import Role
 from metadata.generated.schema.entity.teams.team import Team
 from metadata.generated.schema.entity.teams.user import User
+from metadata.generated.schema.type import basic
+from metadata.generated.schema.type.entityHistory import EntityVersionHistory
+from metadata.generated.schema.type.entityReference import EntityReference
 from metadata.ingestion.ometa.auth_provider import AuthenticationProvider
 from metadata.ingestion.ometa.client import REST, APIError, ClientConfig
-from metadata.ingestion.ometa.mixins.lineageMixin import OMetaLineageMixin
-from metadata.ingestion.ometa.mixins.tableMixin import OMetaTableMixin
+from metadata.ingestion.ometa.mixins.mlmodel_mixin import OMetaMlModelMixin
+from metadata.ingestion.ometa.mixins.table_mixin import OMetaTableMixin
+from metadata.ingestion.ometa.mixins.version_mixin import OMetaVersionMixin
 from metadata.ingestion.ometa.openmetadata_rest import (
     Auth0AuthenticationProvider,
     GoogleAuthenticationProvider,
@@ -32,10 +58,10 @@ from metadata.ingestion.ometa.openmetadata_rest import (
     NoOpAuthenticationProvider,
     OktaAuthenticationProvider,
 )
+from metadata.ingestion.ometa.utils import get_entity_type, uuid_to_str
 
 logger = logging.getLogger(__name__)
 
-
 # The naming convention is T for Entity Types and C for Create Types
 T = TypeVar("T", bound=BaseModel)
 C = TypeVar("C", bound=BaseModel)
@@ -60,7 +86,9 @@ class EntityList(Generic[T], BaseModel):
     after: str = None
 
 
-class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
+class OpenMetadata(
+    OMetaMlModelMixin, OMetaTableMixin, OMetaVersionMixin, Generic[T, C]
+):
     """
     Generic interface to the OpenMetadata API
 
@@ -76,6 +104,7 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
     entity_path = "entity"
     api_path = "api"
     data_path = "data"
+    policies_path = "policies"
     services_path = "services"
     teams_path = "teams"
 
@@ -101,23 +130,26 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
             base_url=self.config.api_endpoint,
             api_version=self.config.api_version,
             auth_header="X-Catalog-Source",
-            auth_token=self._auth_provider.auth_token(),
+            auth_token=lambda: self._auth_provider.get_access_token(),
         )
         self.client = REST(client_config)
         self._use_raw_data = raw_data
 
-    def get_suffix(self, entity: Type[T]) -> str:
+    def get_suffix(self, entity: Type[T]) -> str:  # pylint: disable=R0911,R0912
         """
         Given an entity Type from the generated sources,
         return the endpoint to run requests.
 
         Might be interesting to follow a more strict
         and type-checked approach
+
+        Disabled pylint R0911: too-many-return-statements
+        Disabled pylint R0912: too-many-branches
         """
 
         # Entity Schemas
         if issubclass(
-            entity, get_args(Union[MLModel, self.get_create_entity_type(MLModel)])
+            entity, get_args(Union[MlModel, self.get_create_entity_type(MlModel)])
         ):
             return "/mlmodels"
 
@@ -141,6 +173,16 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         ):
             return "/pipelines"
 
+        if issubclass(
+            entity, get_args(Union[Location, self.get_create_entity_type(Location)])
+        ):
+            return "/locations"
+
+        if issubclass(
+            entity, get_args(Union[Policy, self.get_create_entity_type(Policy)])
+        ):
+            return "/policies"
+
         if issubclass(
             entity, get_args(Union[Table, self.get_create_entity_type(Table)])
         ):
@@ -151,11 +193,6 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         ):
             return "/topics"
 
-        if issubclass(
-            entity, get_args(Union[Model, self.get_create_entity_type(Model)])
-        ):
-            return "/models"
-
         if issubclass(entity, Metrics):
             return "/metrics"
 
@@ -168,6 +205,9 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         if issubclass(entity, Tag):
             return "/tags"
 
+        if issubclass(entity, get_args(Union[Role, self.get_create_entity_type(Role)])):
+            return "/roles"
+
         if issubclass(entity, get_args(Union[Team, self.get_create_entity_type(Team)])):
             return "/teams"
 
@@ -207,32 +247,35 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         ):
             return "/services/pipelineServices"
 
+        if issubclass(
+            entity,
+            get_args(
+                Union[StorageService, self.get_create_entity_type(StorageService)]
+            ),
+        ):
+            return "/services/storageServices"
+
         raise MissingEntityTypeException(
             f"Missing {entity} type when generating suffixes"
         )
 
-    @staticmethod
-    def get_entity_type(
-        entity: Union[Type[T], str],
-    ) -> str:
-        """
-        Given an Entity T, return its type.
-        E.g., Table returns table, Dashboard returns dashboard...
-
-        Also allow to be the identity if we just receive a string
-        """
-        return entity if isinstance(entity, str) else entity.__name__.lower()
-
     def get_module_path(self, entity: Type[T]) -> str:
         """
         Based on the entity, return the module path
         it is found inside generated
         """
 
+        if "policy" in entity.__name__.lower():
+            return self.policies_path
+
         if "service" in entity.__name__.lower():
             return self.services_path
 
-        if "user" in entity.__name__.lower() or "team" in entity.__name__.lower():
+        if (
+            "user" in entity.__name__.lower()
+            or "role" in entity.__name__.lower()
+            or "team" in entity.__name__.lower()
+        ):
             return self.teams_path
 
         return self.data_path
@@ -251,7 +294,6 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         )
 
         class_name = f"Create{entity.__name__}EntityRequest"
-
         create_class = getattr(
             __import__(class_path, globals(), locals(), [class_name]), class_name
         )
@@ -289,8 +331,7 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         """
 
         entity = data.__class__
-        is_create = "create" in entity.__name__.lower()
-        is_service = "service" in entity.__name__.lower()
+        is_create = "create" in data.__class__.__name__.lower()
 
         # Prepare the return Entity Type
         if is_create:
@@ -300,14 +341,7 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
                 f"PUT operations need a CrateEntity, not {entity}"
             )
 
-        # Prepare the request method
-        if is_service and is_create:
-            # Services can only be created via POST
-            method = self.client.post
-        else:
-            method = self.client.put
-
-        resp = method(self.get_suffix(entity), data=data.json())
+        resp = self.client.put(self.get_suffix(entity), data=data.json())
         return entity_class(**resp)
 
     def get_by_name(
@@ -320,13 +354,16 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         return self._get(entity=entity, path=f"name/{fqdn}", fields=fields)
 
     def get_by_id(
-        self, entity: Type[T], entity_id: str, fields: Optional[List[str]] = None
+        self,
+        entity: Type[T],
+        entity_id: Union[str, basic.Uuid],
+        fields: Optional[List[str]] = None,
     ) -> Optional[T]:
         """
         Return entity by ID or None
         """
 
-        return self._get(entity=entity, path=entity_id, fields=fields)
+        return self._get(entity=entity, path=uuid_to_str(entity_id), fields=fields)
 
     def _get(
         self, entity: Type[T], path: str, fields: Optional[List[str]] = None
@@ -343,16 +380,41 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
             return entity(**resp)
         except APIError as err:
             logger.error(
-                f"Error {err.status_code} trying to GET {entity.__class__.__name__} for {path}"
+                f"GET {entity.__name__} for {path}. "
+                + f"Error {err.status_code} - {err}"
             )
             return None
 
+    def get_entity_reference(
+        self, entity: Type[T], fqdn: str
+    ) -> Optional[EntityReference]:
+        """
+        Helper method to obtain an EntityReference from
+        a FQDN and the Entity class.
+        :param entity: Entity Class
+        :param fqdn: Entity instance FQDN
+        :return: EntityReference or None
+        """
+        instance = self.get_by_name(entity, fqdn)
+        if instance:
+            return EntityReference(
+                id=instance.id,
+                type=get_entity_type(entity),
+                name=instance.fullyQualifiedName,
+                description=instance.description,
+                href=instance.href,
+            )
+
+        logger.error(f"Cannot find the Entity {fqdn}")
+        return None
+
     def list_entities(
         self,
         entity: Type[T],
         fields: Optional[List[str]] = None,
         after: str = None,
         limit: int = 1000,
+        params: Dict = {},
     ) -> EntityList[T]:
         """
         Helps us paginate over the collection
@@ -362,16 +424,34 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         url_limit = f"?limit={limit}"
         url_after = f"&after={after}" if after else ""
         url_fields = f"&fields={','.join(fields)}" if fields else ""
+        url_params = f"&{urllib.parse.urlencode(params)}"
+        resp = self.client.get(
+            f"{suffix}{url_limit}{url_after}{url_fields}{url_params}"
+        )
 
-        resp = self.client.get(f"{suffix}{url_limit}{url_after}{url_fields}")
+        if self._use_raw_data:
+            return resp
+
+        entities = [entity(**t) for t in resp["data"]]
+        total = resp["paging"]["total"]
+        after = resp["paging"]["after"] if "after" in resp["paging"] else None
+        return EntityList(entities=entities, total=total, after=after)
+
+    def list_versions(
+        self, entity_id: Union[str, basic.Uuid], entity: Type[T]
+    ) -> EntityVersionHistory:
+        """
+        Version history of an entity
+        """
+
+        suffix = self.get_suffix(entity)
+        path = f"/{uuid_to_str(entity_id)}/versions"
+        resp = self.client.get(f"{suffix}{path}")
 
         if self._use_raw_data:
             return resp
         else:
-            entities = [entity(**t) for t in resp["data"]]
-            total = resp["paging"]["total"]
-            after = resp["paging"]["after"] if "after" in resp["paging"] else None
-            return EntityList(entities=entities, total=total, after=after)
+            return EntityVersionHistory(**resp)
 
     def list_services(self, entity: Type[T]) -> List[EntityList[T]]:
         """
@@ -381,17 +461,17 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         resp = self.client.get(self.get_suffix(entity))
         if self._use_raw_data:
             return resp
-        else:
-            return [entity(**p) for p in resp["data"]]
 
-    def delete(self, entity: Type[T], entity_id: str) -> None:
-        self.client.delete(f"{self.get_suffix(entity)}/{entity_id}")
+        return [entity(**p) for p in resp["data"]]
+
+    def delete(self, entity: Type[T], entity_id: Union[str, basic.Uuid]) -> None:
+        self.client.delete(f"{self.get_suffix(entity)}/{uuid_to_str(entity_id)}")
 
     def compute_percentile(self, entity: Union[Type[T], str], date: str) -> None:
         """
         Compute an entity usage percentile
         """
-        entity_name = self.get_entity_type(entity)
+        entity_name = get_entity_type(entity)
         resp = self.client.post(f"/usage/compute.percentile/{entity_name}/{date}")
         logger.debug("published compute percentile {}".format(resp))
 
@@ -402,6 +482,14 @@ class OpenMetadata(OMetaLineageMixin, OMetaTableMixin, Generic[T, C]):
         resp = self.client.get(f"{self.get_suffix(Tag)}/{category}")
         return [Tag(**d) for d in resp["children"]]
 
+    def create_tag_category(self, data):
+        resp = self.client.post(f"/tags", data=data.json())
+        return [TagCategory(**d) for d in resp["children"]]
+
+    def create_primary_tag_category(self, category, data):
+        resp = self.client.post("/tags/{}".format(category), data=data.json())
+        return [Tag(**d) for d in resp["children"]]
+
     def health_check(self) -> bool:
         """
         Run endpoint health-check. Return `true` if OK
