@@ -56,7 +56,7 @@ class QueryParserProcessor(Processor):
     def process(self, record: TableQuery) -> QueryParserData:
         try:
             start_date = datetime.datetime.strptime(
-                record.analysis_date, "%Y-%m-%d %H:%M:%S"
+                str(record.analysis_date), "%Y-%m-%d %H:%M:%S"
             ).date()
             parser = Parser(record.sql)
             columns_dict = {} if parser.columns_dict is None else parser.columns_dict
