@@ -32,11 +32,14 @@ from metadata.generated.schema.entity.data.table import (
     TableData,
     TableProfile,
 )
+from metadata.generated.schema.entity.tags.tagCategory import Tag
 from metadata.generated.schema.type.entityReference import EntityReference
+from metadata.generated.schema.type.tagLabel import TagLabel
 from metadata.ingestion.api.common import Entity, WorkflowContext
 from metadata.ingestion.api.source import Source, SourceStatus
 from metadata.ingestion.models.ometa_table_db import OMetaDatabaseAndTable
 from metadata.ingestion.models.table_metadata import DeleteTable
+from metadata.ingestion.ometa.client import APIError
 from metadata.ingestion.ometa.ometa_api import OpenMetadata
 from metadata.ingestion.ometa.openmetadata_rest import MetadataServerConfig
 from metadata.ingestion.source.sql_source_common import (
@@ -117,6 +120,10 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                     )
                 return True
         # Catch any errors during profiling init and continue ingestion
+        except ModuleNotFoundError as err:
+            logger.error(
+                f"Profiling not available for this databaseService: {str(err)}"
+            )
         except Exception as exc:  # pylint: disable=broad-except
             logger.debug(traceback.print_exc())
             logger.debug(f"Error loading profiler {repr(exc)}")
@@ -154,6 +161,7 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
             return TableData(columns=cols, rows=rows)
         # Catch any errors and continue the ingestion
         except Exception as err:  # pylint: disable=broad-except
+            logger.debug(traceback.print_exc())
             logger.error(f"Failed to generate sample data for {table} - {err}")
         return None
 
@@ -222,7 +230,6 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                         )
                 # Catch any errors during the profile runner and continue
                 except Exception as err:
-                    logger.error(repr(err))
                     logger.error(err)
                 # check if we have any model to associate with
                 table_entity.dataModel = self._get_data_model(schema, table_name)
@@ -235,7 +242,7 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                     "{}.{}".format(self.config.get_service_name(), table_name)
                 )
             except Exception as err:
-                traceback.print_exc()
+                logger.debug(traceback.print_exc())
                 logger.error(err)
                 self.status.failures.append(
                     "{}.{}".format(self.config.service_name, table_name)
@@ -387,6 +394,7 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
         manifest_columns = mnode.get("columns", {})
         for key in ccolumns:
             ccolumn = ccolumns[key]
+            col_name = ccolumn["name"].lower().replace(".", "_DOT_")
             try:
                 ctype = ccolumn["type"]
                 col_type = ColumnTypeParser.get_column_type(ctype)
@@ -396,7 +404,7 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                 if description is None:
                     description = ccolumn.get("comment", None)
                 col = Column(
-                    name=ccolumn["name"].lower(),
+                    name=col_name,
                     description=description,
                     dataType=col_type,
                     dataLength=1,
@@ -404,7 +412,7 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                 )
                 columns.append(col)
             except Exception as err:  # pylint: disable=broad-except
-                logger.error(f"Failed to parse column type due to {err}")
+                logger.error(f"Failed to parse column {col_name} due to {err}")
 
         return columns
 
@@ -465,27 +473,27 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
         table_columns = []
         columns = inspector.get_columns(table, schema)
         try:
-            for row_order, column in enumerate(columns):
+            for column in columns:
                 try:
-                    col_dict = None
                     if "." in column["name"]:
                         column["name"] = column["name"].replace(".", "_DOT_")
                     children = None
                     data_type_display = None
-                    col_data_length = None
                     arr_data_type = None
                     parsed_string = None
                     if (
                         "raw_data_type" in column
                         and column["raw_data_type"] is not None
                     ):
+
                         column["raw_data_type"] = self.parse_raw_data_type(
                             column["raw_data_type"]
                         )
-                        parsed_string = ColumnTypeParser._parse_datatype_string(
-                            column["raw_data_type"]
-                        )
-                        parsed_string["name"] = column["name"]
+                        if not column["raw_data_type"].startswith(schema):
+                            parsed_string = ColumnTypeParser._parse_datatype_string(
+                                column["raw_data_type"]
+                            )
+                            parsed_string["name"] = column["name"]
                     else:
                         col_type = ColumnTypeParser.get_column_type(column["type"])
                         if col_type == "ARRAY" and re.match(
@@ -507,7 +515,9 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                             col_type = "VARCHAR"
                             data_type_display = "varchar"
                             logger.warning(
-                                f"Unknown type {column['type']} mapped to VARCHAR: {column['name']} {column['type']}"
+                                "Unknown type {} mapped to VARCHAR: {}".format(
+                                    repr(column["type"]), column["name"]
+                                )
                             )
                         col_data_length = (
                             1 if col_data_length is None else col_data_length
@@ -524,7 +534,6 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                             dataTypeDisplay=dataTypeDisplay,
                             dataLength=col_data_length,
                             constraint=col_constraint,
-                            ordinalPosition=row_order,
                             children=children if children else None,
                             arrayDataType=arr_data_type,
                         )
@@ -552,9 +561,31 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
                                 "dataType"
                             ]
                         col_dict = Column(**parsed_string)
+                        try:
+                            if self.config.enable_policy_tags:
+                                if "policy_tags" in column and column["policy_tags"]:
+                                    self.metadata.create_primary_tag_category(
+                                        category=self.config.tag_category_name,
+                                        data=Tag(
+                                            name=column["policy_tags"], description=""
+                                        ),
+                                    )
+                        except APIError:
+                            if column["policy_tags"] and self.config.enable_policy_tags:
+                                col_dict.tags = [
+                                    TagLabel(
+                                        tagFQN=f"{self.config.tag_category_name}.{column['policy_tags']}",
+                                        labelType="Automated",
+                                        state="Suggested",
+                                    )
+                                ]
+                        except Exception as err:
+                            logger.debug(traceback.print_exc())
+                            logger.debug(err)
+
                         om_column = col_dict
                 except Exception as err:
-                    logger.error(traceback.print_exc())
+                    logger.debug(traceback.print_exc())
                     logger.error(f"{err} : {column}")
                     continue
                 table_columns.append(om_column)
@@ -564,13 +595,11 @@ class SQLSource(Source[OMetaDatabaseAndTable]):
             return None
 
     def _check_col_length(self, datatype, col_raw_type):
-        if datatype is not None and datatype.upper() in {
-            "CHAR",
-            "VARCHAR",
-            "BINARY",
-            "VARBINARY",
-        }:
-            return col_raw_type.length if col_raw_type.length else 1
+        if datatype and datatype.upper() in {"CHAR", "VARCHAR", "BINARY", "VARBINARY"}:
+            try:
+                return col_raw_type.length if col_raw_type.length else 1
+            except AttributeError:
+                return 1
 
     def run_data_profiler(self, table: str, schema: str) -> TableProfile:
         """
